/*
 * ProGuardCORE -- library to process Java bytecode.
 *
 * Copyright (c) 2002-2020 Guardsquare NV
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package proguard.classfile.kotlin.visitor;

import proguard.classfile.Clazz;
import proguard.classfile.kotlin.*;

/**
 * This KotlinMetadataVisitor lets a given KotlinFunctionVisitor visit all functions
 * of visited KotlinMetadata.
 */
public class AllFunctionsVisitor
implements   KotlinMetadataVisitor
{
    private final KotlinFunctionVisitor[] delegateFunctionVisitors;

    public AllFunctionsVisitor(KotlinFunctionVisitor ... kotlinFunctionVisitor)
    {
        this.delegateFunctionVisitors = kotlinFunctionVisitor;
    }

    @Override
    public void visitAnyKotlinMetadata(Clazz clazz, KotlinMetadata kotlinMetadata)
    {
    }

    @Override
    public void visitKotlinDeclarationContainerMetadata(Clazz                              clazz,
                                                        KotlinDeclarationContainerMetadata kotlinDeclarationContainerMetadata)
    {
        for (KotlinFunctionVisitor functionVisitor : delegateFunctionVisitors)
        {
            kotlinDeclarationContainerMetadata.functionsAccept(clazz, functionVisitor);
        }
    }

    @Override
    public void visitKotlinSyntheticClassMetadata(Clazz                            clazz,
                                                  KotlinSyntheticClassKindMetadata kotlinSyntheticClassKindMetadata)
    {
        for (KotlinFunctionVisitor functionVisitor : delegateFunctionVisitors)
        {
            kotlinSyntheticClassKindMetadata.functionsAccept(clazz, functionVisitor);
        }
    }
}
